;===============================================
; NEO2.2 Disassembly by cableiso 2/2/2
; Parent File: neo2_2US.hex
; Format: PIC Assembly
; Host Processor: Microchip PIC 12C508
; Albums listened to: Gang Starr-Moment of Truth,
; Dieselboy-The 6ixth Session, J.Smooth-On the block,
; hustlin', Jaybird [untitled underground mixtape-bangin!]
;-----------------------------------------------
; Well here's another installment of modchip
; source code - hope it continues to help advance 
; the scene. Again this was hand-disassembled and 
; hand-commented from the original hex.  It has 
; been verified to compile to the original hex, 
; (except for all the 0's at the end, of course)
; and has been simulated to extract timings. My
; comments may contain errors, but the code is
; solid.
;
; Email ?'s/comments/corrections to: 
; cableiso@hotmail.com
;
; Requesting: PS2/DVD/DC/etc HEX files, as well
; as serious discussion about modchip development.
; ie-I'll crack the Messiah if you get me one 
;-----------------------------------------------
; neo 2.2 - Tiny Pinout & I/O Description
; 	   Using PIC 12c508
;                   /---\/---\
; (+V)         Vdd -|1     8|- Vss    (GND)
; (       ) >> GP5 -|2      7|- GP0 << (       )
; (CD Stop) << GP4 -|3      6|- GP1 >> (Eject? )
; (Reset? ) << GP3 -|4      5|- GP2 >> (Data)
;                   \--------/
;===============================================


;===============================================
;-----------Fuses & Compiler Settings-----------
;===============================================


	__FUSES _MCLRE_OFF & _CP_OFF & _WDT_OFF & _IntRC_OSC
	list    p=12c508
	#include <p12c508.inc>


;===============================================
;-----------------Variables---------------------
;===============================================
; These locations are hardcoded here, so this 
; file compiles the same as neo2_2us.hex

g	equ	0x7
h	equ	0x9
i	equ	0xa
j	equ	0x10
k	equ	0x11
l	equ	0x12
m	equ	0x13
n	equ	0x14
o	equ	0x15
p	equ	0x17
q	equ	0x19
r	equ	0x1f


;===============================================
;---------------------Code----------------------
;===============================================

; -Program Entry Point-
	movwf	OSCCAL     	; 
	goto	Start		; 

Copyright
	movf	g,W   		; Load index into table
	addwf	PCL, F   	; Grab the letter & return

	dt 	'C','O','P','Y','R','I','G','H','T'
	retlw  	0xB8    ; -the copyright sign-
	dt 	'2','0','0','0','>','2','0','0','1','.'
	dt 	'P','L','A','Y','S','T','A','Y','S','T'
	dt 	'I','O','N','M','O','D','S','.','C','O'
	dt 	'M','.','.','&','M','R','.','T','@','D'
	dt	'V','D','M','O','D','S','.','C','O','.'
	dt	'U','K'


;===============================================
; Start - The neo 2.2's main routine
;-----------------------------------------------
; Called only by $0001 (Jump Vector)
;===============================================
Start:

	clrf	p    		; Clear a few variables
	clrf	q    		; 
	clrf	r    		; 
	clrf	i     		; 
	goto 	Anti_Lamer	; Check to see if copyright was altered


L2
	clrf   GPIO    		; Set all port buffers to 0
	movlw  0xC7    		; Set chip configuration to
	option         		; TMR0 internal, pullups off
	movlw  0xFF    		; Set all I/O pins as 
	tris    GPIO  		; input/tristate

L3
	btfss  GPIO,0x5 	; Loop here until GP5 (pin 2)
	goto   L3    		; goes HIGH

L4
	btfsc  GPIO,0x3		; Now loop here until GP3 (pin 4)
	goto   L4    		; goes LOW
	movlw  0x14    		; Delay for .6 sec (20 * 32.778ms)
	movwf  p    		
	call   Timer_Delay    	
	btfsc  GPIO,0x3		; If GP3 (Pin 4) is HIGH, start sending immediately
	goto   Send_Loop   	; 
	movlw  0xC8    		; Delay for 6.555 sec (200 * 32.778ms)
	movwf  p    		 
	call   Timer_Delay    	 
	btfsc  GPIO,0x3		; If GP3 (PIN 4) is high, go back to Start & repeat
	goto   Start		 

L5
	btfss  GPIO,0x3		; Loop here until GP3 (pin 4) is HIGH
	goto   L5    		; 
	movlw  0xE9    		; Set GP 1,2, & 4 as output, all 
	tris    GPIO  		; others input/tristate
	movlw  0x2C    		; Load i with 44
	movwf  i     		; 
	bsf    GPIO,0x4		; Set GP4 (Pin 3) HIGH
	movlw  0x64    		; 
	movwf  p    		; 
	call   Timer_Delay    	; 

L6
	bsf    GPIO,0x4		; Set GP4 (Pin 3) HIGH
	bsf    GPIO,0x1		; Set GP1 (Pin 6) HIGH
	bcf    GPIO,0x2		; Set GP2 (Pin 5) LOW
	movlw  0x17    		; Wait about 18.5 ms
	movwf  r    		; 
	call   Little_Delay    	; 
	bcf    GPIO,0x4		; Set GP4 (Pin 3) Low
	movlw  0x26    		; Delay 30.7ms (38 * .808)
	movwf  r    		
	call   Little_Delay
	decfsz i, F    		; 
	goto   L6    		; 
	movlw  0xFF    		; Tristate all I/O pins
	tris   GPIO  		
	movlw  0x5A    		; Wait about 3 sec (2.95 = 90 * 32.778ms)
	movwf  p    		; 
	call   Timer_Delay    	; 
	movlw  0x2D    		; Load pulse counter with 45
	movwf  i     		 
	movlw  0xE9    		; Set GP 1,2, & 4 as output, all
	tris   GPIO  		; others input/tristated

L7
	bsf    GPIO,0x4 	; Set GP4 (Pin 3) HIGH
	bcf    GPIO,0x1 	; Set GP1 (Pin 6) LOW
	bsf    GPIO,0x2 	; Set GP2 (Pin 5) HIGH
	movlw  0x1E    		; Delay 24.24ms (30 * .808)
	movwf  r    		 
	call   Little_Delay
	bcf    GPIO,0x4 	; Set GP4 (Pin 3) LOW
	movlw  0x19    		; Delay 20.2ms (25 * .808)
	movwf  r    		 
	call   Little_Delay
	decfsz i, F    		; Decrement pulse counter
	goto   L7    		; Repeat if counter !=0
	movlw  0xFF    		; Else tristate all I/O pins
	tris   GPIO  		
	goto   Send_Loop	; And go send our data to the ps2


;===============================================
; Inf_Loop_1 and 2: Throw the chip into an 
; infinite loop so it's useless. Inf_Loop_1
; is never really called, only Inf_Loop_2
;-----------------------------------------------
; Called by: Anti_Lamer
;===============================================

Inf_Loop_1
	goto   Inf_Loop_1      	; Infinite Loop

Inf_Loop_2
	nop            		; 
	goto   Inf_Loop_2      	; Another Infinite Loop


;===============================================
; Little_Delay: Delay for (r * .808)ms
;-----------------------------------------------
; Called by: Start
;===============================================
Little_Delay
	clrf   q    	; 

L10
	movlw  0xC8    	; Load counter with 200
	movwf  q    	 

L11
	nop            	; Waste 4 cycles 200 times
	decfsz q, F    	; to get .8ms
	goto   L11    	; 
	decfsz r, F   	; Have we done all r cycles?
	goto   L10    	; if not, loop again
	retlw  0x0     	; Else return to caller
;-----------------------------------
	movlw  0x32    	; ??? Some junk


;===============================================
; Delay_W_ms: Delay for W milliseconds.  Actual
; delay time is about 1.01ms so for large values
; of W (W > 50) take this into consideration
;-----------------------------------------------
; Called by: Send_W_Phrases
;===============================================
Delay_W_ms
	movwf  j    	; Store # of ms into j

L12
	movlw  0xFA    	; Load up a counter w/ 250
	movwf  k    	 

L13
	nop            	; These couple instructions take a 
	decfsz k, F    	; total of 4us each time, so
	goto   L13    	; that's where we get our 1ms
	decfsz j, F    	; More milliseconds or are we done?
	goto   L12    	; If more time, repeat the cycle
	retlw  0x0     	; Else return to caller


;===============================================
; Send_W_Phrases: Send multiple "SCEA" messages
; to the ps2.  Number of phrases to send is 
; passed in W register
;-----------------------------------------------
; Called by: Send_Loop
;===============================================

Send_W_Phrases
	clrf   p    	; Clear a variable
	movwf  p    	; Now save our number of phrases to send

L14
	movlw  0xA    	; delay about 75ms (74 * 1.01ms)
	call   Delay_W_ms    	 
	movlw  0x4     	; load our counter into r 
	movwf  r    	; (4 letters per phrase)

L15
	movf   m,W  	; Restore index
	call   Messages	; Go get next letter of copyright message in W
	movwf  l    	; Move that letter to l
	comf   l, F    	; And invert the bits
	movlw  0x8     	; 8 bits in a byte, here we
	movwf  h     	; load up our bit counter
	movlw  0xFF    	; Tristate all port pins
	tris   GPIO  	 
	movlw  0x4     	; delay 4ms
	call   Delay_W_ms

L16
	rrf    l, F    	; Roll a bit out of buffer & into the Carry flag
	bcf    0x6,0x5 	; Set GP5 buffer to LOW
	btfsc  STATUS,C	; If carry flag is low, 
	movlw  0xFF    	; We'll tristate all ports 
	btfss  STATUS,C	; If carry flag is high
	movlw  0xDF    	; Set GP5 as an output (it'll be low)
	tris    GPIO  	; Set ports
	movlw  0x4     	; load up a 4ms delay
	call   Delay_W_ms    	
	decfsz h, F    	; Decrement bit counter & continue sending
	goto   L16    	; if bit counter != 0
	movlw  0xDF    	; Else fire up some stop bits..
	tris    GPIO  	; Set GP5 as an output low
	movlw  0x8     	; Load up an 8ms delay
	call   Delay_W_ms
	incf   m, F    	; Increment our phrase index
	decfsz r, F    	; decrement letter counter - All letters sent?
	goto   L15    	; If not, go do next letter
	decfsz p, F    	; If so, decrement phrase counter - All phrases sent?
	goto   L14    	; If not, go send next phrase
	retlw  0x3     	; If all phrases done, return to caller
;-----------------------------------

;===============================================
; Messages: Return letters of the sony messages
; for transmission to the ps2
;-----------------------------------------------
; Called by: Send_W_Phrases
;===============================================
Messages
	addwf	PCL, F  	; Add w to PC & return that letter
	dt 	'S','C','E','A'
	dt	'S','C','E','A'
	dt	'S','C','E','A'

;-----------------------------------

	nop            	; 
	nop            	; 
	dt	'P','L','A','Y','S','T','A','Y','T','I'
	dt	'O','M','O','D','S','.','C','O','M','A'
	dt	'N','D','_','M','R','T'

;===============================================
; Anti_Lamer: This routine XORs letters 2-32 of
; the Copyright located near the Program Entry
; Point.  If total is not equal to $FA, we jump
; to an infinite loop.  In other words, if you're
; a dumbass who hex edits his own name on top of
; other peoples, you will burn useless chips!
;-----------------------------------------------
; Called by: Start  
;===============================================
Anti_Lamer
	clrf   o    		; Clear some vars
	clrf   g     		 
	clrf   n    		 
	movlw  0x2     		; Start our letter counter with letter #2
	movwf  g     		; load pointer to next letter
	movlw  0x1E    		; We're going to sum (XOR) 30 characters
	movwf  o    		; so let's load counter

Anti_Lamer_loop
	call   Copyright	; Get next letter in W
	xorwf  n, F    		; Add it to the rolling sum (n=n XOR W)
	incf   g, F    		; increment pointer
	bcf    0x3,0x2 		; Reset a Zero Flag that may have resulted
	bcf    0x3,0x0 		; Reset a Carry Flag that may have resulted
	decfsz o, F    		; decrement counter
	goto   Anti_Lamer_loop	; Repeat if counter != 0
	movlw  0xFA    		; Load W with FA (The correct Checksum)
	xorwf  n,W  		; XOR our Checksum with $FA
	btfss  0x3,0x2 		; IF answer != 0 then checksums don't match - copyright altered!
	goto   Inf_Loop_2	; Punish those hex editing monkeys with an infinite loop 
	goto   L2    		; Else continue on like a nice modchip

;===============================================
; Timer Delay: Waste (p * 32.778)ms
;-----------------------------------------------
; Called by: Start, Send_Loop
;===============================================
Timer_Delay
	clrf   TMR0    		; Clear the Timer
L20
	btfss  TMR0,0x7		; Loop here until timer > 127
	goto   L20   		 
	decfsz p, F    		; Decrement p 
	goto   Timer_Delay	; Loop again if necessary
	retlw  0x0     		; else return

;===============================================
; Send_Loop: Our final destination - churn out
; the Sony Copyright messages over & over
;-----------------------------------------------
; Called by: Start
;===============================================
Send_Loop
	movlw  0xFF    		; Tristate all GPIO ports
	tris   GPIO		
	movlw  0xF     		; Delay for about 1/2 sec (15 x 32.778 = 491ms)
	movwf  p    		 
	call   Timer_Delay    	 
	btfsc  GPIO,0x0 	; If GP0 (pin 7) is HIGH, go delay again
	goto   Send_Loop   	 
	clrf   m    		; IF LOW, send our 3 'SCEA' phrases
	movlw  0x3     		; to the ps2
	call   Send_W_Phrases    		
	goto   Send_Loop   	; And repeat


	end			; That's it
